
try:
    # Local development
    from software.firmware.europi import ain, b1, b2, clamp, cvs, din, k1, k2, oled
    from software.firmware.europi import MAX_OUTPUT_VOLTAGE, OLED_HEIGHT, OLED_WIDTH
    from software.firmware.europi_script import EuroPiScript
except ImportError:
    # Device import path
    from europi import *
    from europi_script import EuroPiScript

from time import sleep_ms, ticks_diff, ticks_ms

HEADER_DURATION = 2000  # 2 seconds in ms


class RadioScanner(EuroPiScript):
    def __init__(self):
        super().__init__()

        # Load state if previous state exists.
        state = self.load_state_json()
        # Set state variables with default fallback values if not found in the
        # json save state.
        self.knob_mapping = state.get("knob_mapping", 0)
        self.cv_mapping = state.get("cv_mapping", [0, 1, 2, 3, 4, 5])

        self.knob_mapping_text = ['Off', 'Knob 1', 'Knob 2']

        def remap_knob():
            self.knob_mapping += 1
            if self.knob_mapping == 3:
                self.knob_mapping = 0
            self.save_state()

        def rotate_cvs():
            self.cv_mapping = self.cv_mapping[1:] + [self.cv_mapping[0]]
            self.save_state()

        b1.handler(rotate_cvs)
        din.handler(rotate_cvs)
        b2.handler(remap_knob)

    def save_state(self):
        """Save the current state variables as JSON."""
        state = {
            "knob_mapping": self.knob_mapping,
            "cv_mapping": self.cv_mapping,
        }
        self.save_state_json(state)

    def value_to_cv(self, value):
        return value * MAX_OUTPUT_VOLTAGE

    def x_to_oled(self, x):
        return round(x * (OLED_WIDTH - 1))

    def y_to_oled(self, y):
        return (OLED_HEIGHT - 1) - round(y * (OLED_HEIGHT - 1))

    def do_step(self, x, y):
        oledx = self.x_to_oled(x)
        oledy = self.y_to_oled(y)

        cvx = self.value_to_cv(x)
        cvy = self.value_to_cv(y)

        oled.fill(0)
        oled.vline(oledx, 0, OLED_HEIGHT, 1)
        oled.hline(0, oledy, OLED_WIDTH, 1)

        cvs[self.cv_mapping[0]].voltage(cvx)  # 0 to 10 (volts)
        cvs[self.cv_mapping[1]].voltage(cvy)
        cvs[self.cv_mapping[2]].voltage(abs(cvy - cvx))
        cvs[self.cv_mapping[3]].voltage(MAX_OUTPUT_VOLTAGE - cvx)
        cvs[self.cv_mapping[4]].voltage(MAX_OUTPUT_VOLTAGE - cvy)
        cvs[self.cv_mapping[5]].voltage(MAX_OUTPUT_VOLTAGE - abs(cvy - cvx))

        sleep_ms(10)

    def display_mapping(self, new_map):
        oled.fill_rect(0, 0, 64, 12, 1)
        oled.text(self.knob_mapping_text[new_map], 0, 2, 0)

    def main(self):

        while True:
            if self.knob_mapping != 1:
                x = k1.percent()
            else:
                x = clamp(ain.percent() + k1.percent(), 0, 1)

            if self.knob_mapping != 2:
                y = k2.percent()
            else:
                y = clamp(ain.percent() + k2.percent(), 0, 1)

            self.do_step(x, y)

            if ticks_diff(ticks_ms(), b2.last_pressed()) < HEADER_DURATION:
                self.display_mapping(self.knob_mapping)

            oled.show()

if __name__ == "__main__":
    RadioScanner().main()



from machine import ADC
from time import sleep

from europi import OLED_HEIGHT, OLED_WIDTH, ain, b1, b2, cv1, cv2, cv3, cv4, cv5, cv6, din, k1, k2, oled
from europi_script import EuroPiScript

"""
A diagnostic utility intended to help prove out a new EuroPi build and calibration. Each aspect of the EuroPi's hardware
is exercised.

- din: value displayed on screen
- ain: value displayed on screen
- b1: rotate output voltages backwards
- b2: rotate output voltages forwards
- k1: value 0-99 displayed on screen
- k2: value 0-99 displayed on screen
- cvX: output a constant voltage, one of [0, 0.5, 1, 2.5, 5, 10]
"""

TEMP_CONV_FACTOR = 3.3 / 65535

class Diagnostic(EuroPiScript):

    def __init__(self):
        super().__init__()
        self.temp_sensor = ADC(4)
        self.voltages = [
            0,  # min
            0.5,  # not 0 but still below DI's threshold
            1,
            2.5,
            5,
            10,  # max
        ]


    def calc_temp(self):
        # see the pico's datasheet for the details of this calculation
        return 27 - ((self.temp_sensor.read_u16() * TEMP_CONV_FACTOR) - 0.706) / 0.001721

    @staticmethod
    def convert_fahrenheit(temp_c):
        return (temp_c * 1.8) + 32


    def rotate_r(self):
        self.voltages = self.voltages[-1:] + self.voltages[:-1]


    def rotate_l(self):
        self.voltages = self.voltages[1:] + self.voltages[:1]


    def main(self):

        b1.handler(self.rotate_l)
        b2.handler(self.rotate_r)

        while True:

            # Set the outputs to useful values
            cv1.voltage(self.voltages[0])
            cv2.voltage(self.voltages[1])
            cv3.voltage(self.voltages[2])
            cv4.voltage(self.voltages[3])
            cv5.voltage(self.voltages[4])
            cv6.voltage(self.voltages[5])

            oled.fill(0)

            # calc and format temp
            use_fahrenheit = b1.value() or b2.value()
            t = self.calc_temp()
            formatted_temp = f"{int(self.convert_fahrenheit(t) if use_fahrenheit else t)}{'F' if use_fahrenheit else 'C'}"

            # display the input values
            oled.text(f"ain: {ain.read_voltage():5.2f}v {formatted_temp}", 2, 3, 1)
            oled.text(f"k1: {k1.read_position():2}  k2: {k2.read_position():2}", 2, 13, 1)
            oled.text(f"din:{din.value()} b1:{b1.value()} b2:{b2.value()}", 2, 23, 1)

            # show the screen boundaries
            oled.rect(0, 0, OLED_WIDTH, OLED_HEIGHT, 1)
            oled.show()

            sleep(0.1)


if __name__ == "__main__":
    Diagnostic().main()



from europi import *
import machine
from europi_script import EuroPiScript
from time import ticks_diff, ticks_ms
from math import fabs, floor
from random import choice

'''
Strange Attractor
author: Sean Bechhofer (github.com/seanbechhofer)
date: 2022-03-15
labels: gates, triggers, randomness

Strange Attractor is a source of chaotic modulation. It can use a variety of different implementations.

Outputs 1, 2 and 3 are based on the x, y and z values generated by the attractor.

Outputs 4, 5 and 6 are gates based on the values of x, y and z and relationships between them. 

digital_in: Pause motion when HIGH
analog_in: 

knob_1: Adjust speed
knob_2: Adjust threshold for triggers

button_1: Decrease output voltage range, change equation system
button_2: Increase output voltage range

output_1: x 
output_2: y
output_3: z
output_4: triggers/gates
output_5: triggers/gates
output_6: triggers/gates

'''
# Version number

VERSION="1.0"

# Overclock the Pico for improved performance.
machine.freq(250_000_000)

# Maximum voltage output. Cranking this up may cause issues with some modules.
MAX_OUTPUT = MAX_OUTPUT_VOLTAGE

'''
Implementation of strange attractors, providing chaotic values for modulation.

* Lorenz. Well known system of equations giving chaotic behaviour. 
  See https://en.wikipedia.org/wiki/Lorenz_system
* Pan-Xu-Zhou. 
  See https://www.semanticscholar.org/paper/Controlling-a-Novel-Chaotic-Attractor-using-Linear-Pan-Xu/72f9c1b1f892b3aeea26af330d44011a20250f32
* Rikitake. Used to model the earth's geomagnetic field and explain the irregular switch in polarity. 
  See Llibre, Jaume & Messias, Marcelo. (2009). Global dynamics of the Rikitake system. Physica D: Nonlinear Phenomena. 238. 241-252. 10.1016/j.physd.2008.10.011. 
* Rossler. Use with caution. The z co-ord sits around zero for long periods. 
  See https://en.wikipedia.org/wiki/R%C3%B6ssler_attractor
'''

class Attractor:
    def __init__(self, point=(0.,1.,1.05), dt=0.01, name="Attractor"):
        self.initial_state = point
        self.x = point[0]
        self.y = point[1]
        self.z = point[2]
        self.dt = dt
        self.name = name
        self.x_min = self.x
        self.y_min = self.y
        self.z_min = self.z
        self.x_max = self.x
        self.y_max = self.y
        self.z_max = self.z
        # arbitrary initial range values    
        self.x_range = 100
        self.y_range = 100
        self.z_range = 100
        
    # The range of values produced depends on the parameters and the
    # specifics of the equations. If we know the range, we can then
    # normalise coordinates for use when generating CV. This method
    # runs through a number of iterations to estimate ranges.
    def estimate_ranges(self,steps=100000):
    
        # Execute a number of steps to get upper and lower bounds. 
        for i in range(steps):
            self.step()
            
            self.x_max = max(self.x, self.x_max)
            self.y_max = max(self.y, self.y_max)
            self.z_max = max(self.z, self.z_max)
            self.x_min = min(self.x, self.x_min)
            self.y_min = min(self.y, self.y_min)
            self.z_min = min(self.z, self.z_min)

        self.x_range = self.x_max-self.x_min
        self.y_range = self.y_max-self.y_min
        self.z_range = self.z_max-self.z_min
        
        # Reset to initial parameters
        self.x = self.initial_state[0]
        self.y = self.initial_state[1]
        self.z = self.initial_state[2]
        

    def x_scaled(self):
        return (100.0 * (self.x - self.x_min))/self.x_range

    def y_scaled(self):
        return (100.0 * (self.y - self.y_min))/self.y_range

    def z_scaled(self):
        return (100.0 * (self.z - self.z_min))/self.z_range

    def __str__(self):
        return (f"{self.name:>16} ({self.x:2.2f},{self.y:2.2f},{self.z:2.2f})({self.x_scaled():2.2f},{self.y_scaled():2.2f},{self.z_scaled():2.2f})")
    
    def step(self):
        '''
        Update the point. This needs to be implemented in subclasses. 
        '''
        pass

'''
Implementation of a simple Lorenz Attractor, see 

https://en.wikipedia.org/wiki/Lorenz_system

Default uses well known values of s=10,r=28,b=2.667. 
'''
class Lorenz(Attractor):
    def __init__(self, point=(0.,1.,1.05), params=(10,28,2.667), dt=0.01):
        super().__init__(point, dt, "Lorenz")
        self.s = params[0]
        self.r = params[1]
        self.b = params[2]

    def step(self):
        '''
        Update the point.
        '''
        x_dot = self.s*(self.y - self.x)
        y_dot = self.r*self.x - self.y - self.x*self.z
        z_dot = self.x*self.y - self.b*self.z
        self.x += x_dot * self.dt
        self.y += y_dot * self.dt
        self.z += z_dot * self.dt
        
# Pan-Xu-Zhou
'''
Implementation of Pan-Xu-Zhou
'''
class PanXuZhou(Attractor):
    def __init__(self, point=(1.,1.,1.), params=(10.0,2.667,16.0), dt=0.01):
        super().__init__(point,dt, "Pan-Xu-Zhou")
        self.a = params[0]
        self.b = params[1]
        self.c = params[2]

    def step(self):
        '''
        Update the point.
        '''
        x_dot = self.a*(self.y - self.x)
        y_dot = self.c*self.x - self.x*self.z
        z_dot = self.x*self.y - self.b*self.z
        self.x += x_dot * self.dt
        self.y += y_dot * self.dt
        self.z += z_dot * self.dt

'''
Implementation of Rossler. The z co-rd spends a lot of time around zero, so use with caution.
'''
class Rossler(Attractor):
    def __init__(self, point=(0.1,0.0,-0.1), params=(0.13,0.2,6.5), dt=0.01):
        super().__init__(point,dt, "Rossler")
        self.a = params[0]
        self.b = params[1]
        self.c = params[2]

    def step(self):
        '''
        Update the point.
        '''
        x_dot = -(self.y + self.z)
        y_dot = self.x + self.a*self.y
        z_dot = self.b + self.z*(self.x - self.c)
        self.x += x_dot * self.dt
        self.y += y_dot * self.dt
        self.z += z_dot * self.dt

'''
Implementation of Rikitake. 
'''
class Rikitake(Attractor):
    def __init__(self, point=(0.1,0.0,-0.1), params=(5.0,2.0), dt=0.01):
        super().__init__(point,dt, "Rikitake")
        self.a = params[0]
        self.mu = params[1]

    def step(self):
        '''
        Update the point.
        '''
        x_dot = -(self.mu * self.x) + (self.z*self.y)
        y_dot = -(self.mu * self.y) + self.x*(self.z - self.a)
        z_dot = 1 - (self.x * self.y)
        self.x += x_dot * self.dt
        self.y += y_dot * self.dt
        self.z += z_dot * self.dt


def get_attractors():
    return [Lorenz(), PanXuZhou(), Rikitake(), Rossler()]

class StrangeAttractor(EuroPiScript):
    def __init__(self):
        # Initialise and calculate ranges. 
        # This will take around 30 seconds per attractor.
        self.attractors = get_attractors()
        for att in self.attractors:
            self.initialise_message(att.name)
            att.estimate_ranges()
            self.done_message()
        # select a random attractor
        self.selected_attractor = choice(range(0,len(self.attractors)))
        self.a = self.attractors[self.selected_attractor]
        # Initialize variables
        self.checkpoint = 0
        # time before update
        self.period = 100
        # output range.
        self.range = MAX_OUTPUT
        # initial threshold for gates
        self.threshold = 20
        # freeze motion
        self.freeze = False
        # Display details
        self.show_detail = True

        # Triggered when button 1 is released
        # Short press: decrease range
        # Long press: change equation system
        @b1.handler_falling
        def b1Pressed():
            if ticks_diff(ticks_ms(), b1.last_pressed()) >  300:
                # long press This will result in a jump in parameters
                # as each attractor has its own x,y,z
                # coordinates. Possible improvement is to share or set
                # coordinates on change.
                self.selected_attractor = (self.selected_attractor + 1) % len(self.attractors)
                self.a = self.attractors[self.selected_attractor]
            else:
                # short press
                self.range -= 1
                if self.range < 1:
                    self.range = 1

        # Triggered when button 2 is released.
        # Short press: increase range
        # Long press: toggle display
        @b2.handler_falling
        def b2Pressed():
            
            if ticks_diff(ticks_ms(), b2.last_pressed()) >  300:
                # long press
                self.show_detail = not self.show_detail
            else:
                # short press
                self.range += 1
                if self.range > MAX_OUTPUT:
                    self.range = MAX_OUTPUT

        # Freeze is triggered when din goes HIGH.
        @din.handler
        def dinTrigger():
            # Pause
            self.freeze = True

        @din.handler_falling
        def dinTriggerEnd():
            # Start agin
            self.freeze = False

    def update_values(self):
        if not self.freeze:
            self.a.step()
        
    def update_speed(self):
        # Set speed based on the knob.
        # The range is piecewise linear from fully CCW to noon and noon to fully CW.
        # TODO: allow speed adjustment via CV. 
        val = k1.read_position()
        low = 1000 # CCW
        mid = 100 # noon
        high = 10 # CW

        if val == 0:
            self.period = low
        elif val < 50:
            self.period = low - ((low-mid) * (val/50))
        else:
            self.period = mid - ((mid-high) * (val-50)/50)
        

    def update_threshold(self):
        self.threshold = k2.read_position(steps=41)        

    def update(self):
        # Change the values and output
        self.update_values()
        cv1.voltage((self.range * self.a.x_scaled()) / 100)
        cv2.voltage((self.range * self.a.y_scaled()) / 100)
        cv3.voltage((self.range * self.a.z_scaled()) / 100)
        # Calculate gates
        # gate 1 fires if x is divisible by 2 when considered an int
        self.gate4 = floor(self.a.x_scaled()) % 2 == 0
        # gates 2 and 3 look at the differences between the outputs. 
        self.gate5 = fabs(self.a.y_scaled() + self.a.z_scaled() - 2*self.a.x_scaled()) > self.threshold
        self.gate6 = fabs(self.a.z_scaled() + self.a.x_scaled() - 2*self.a.y_scaled()) > self.threshold

        # Set gates
        cv4.value(self.gate4)
        cv5.value(self.gate5)
        cv6.value(self.gate6)
        
        self.checkpoint = ticks_ms()
        self.update_screen()

    def main(self):
        while True:
            self.update_speed()
            self.update_threshold()
            if ticks_diff(ticks_ms(), self.checkpoint) > self.period:
                self.update()

    def initialise_message(self, att_name):
        oled.fill(0)
        oled.text('Strange',0,0,1)
        oled.text(f"Attractor v{VERSION}",0,8,1)
        oled.text('Initialising...',0,16,1)
        oled.text(att_name,10,24,1)
        oled.show()
        
    def done_message(self):
        oled.fill(0)
        oled.text('Done',0,0,1)
        oled.show()
        
    def update_screen(self):
        oled.fill(0)
        if self.show_detail:
            oled.text('1:' + str(int(self.a.x_scaled())),0,0,1)
            oled.text('2:' + str(int(self.a.y_scaled())),0,8,1)
            oled.text('3:' + str(int(self.a.z_scaled())),0,16,1)
            oled.text('S:' + str(int(self.period)),40,0,1)
            oled.text('T:' + str(int(self.threshold)),40,8,1)
            oled.text('R:' + str(int(self.range)),40,16,1)
        else:
            oled.text('1:',0,0,1)
            oled.fill_rect(20,0,int(0.75*self.a.x_scaled()),6,1)
            oled.rect(20,0,75,6,1)
            oled.text('2:',0,8,1)
            oled.fill_rect(20,8,int(0.75*self.a.y_scaled()/2),6,1)
            oled.rect(20,8,75,6,1)
            oled.text('3:',0,16,1)
            oled.fill_rect(20,16,int(0.75*self.a.z_scaled()/2),6,1)
            oled.rect(20,16,75,6,1)

        if self.gate4:
            oled.text('4',100,0,1)
        if self.gate5:
            oled.text('5',100,8,1)
        if self.gate6:
            oled.text('6',100,16,1)
        if self.freeze:
            oled.text('FREEZE',0,24,1)
        oled.text(self.a.name,55,24,1)
            
        oled.show()

if __name__ == "__main__":
    StrangeAttractor().main()



from europi import *
import machine
from europi_script import EuroPiScript
from time import ticks_diff, ticks_ms

'''
NoddyHolder
author: Sean Bechhofer (github.com/seanbechhofer)
date: 2022-03-26
labels: sample & hold, trigger & hold

Two channels of sample and hold/trigger and hold. 

Digital in: Gate
Analog in: CV

Output 1: Gate
Output 2: Sample and Hold based on Gate and CV
Output 3: Trigger and Hold based on Gate and CV

Output 4: Inverted Gate
Output 5: Sample and Hold based on Inverted Gate and CV
Output 6: Trigger and Hold based on Inverted Gate and CV

digital_in: gate
analog_in: cv

knob_1: Not used
knob_2: Not used

button_1: Not used
button_2: Not used

output_1: gate 
output_2: s&h
output_3: t&h
output_4: inverted gate
output_5: s&h using inverted gate
output_6: t&h using inverted gate

'''
# Version number

VERSION="1.0"


class NoddyHolder(EuroPiScript):
    def __init__(self):
        # Overclock the Pico for improved performance.
        machine.freq(250_000_000)

        self.gate = False
        # Keep track of values for display, S&H and T&H for each
        # channel
        self.channel_1 = [0,-1]
        self.channel_2 = [0,-1]
        
        # Triggered when din goes HIGH.
        @din.handler
        def dinTrigger():
            self.gate = True
            # Sample input
            sample = ain.read_voltage()
            # Set mirrored gate HIGH
            cv1.on()
            # Set inverse gate LOW
            cv4.off()

            # Set S&H to sample
            cv2.voltage(sample)
            self.channel_1[0] = sample
            # Channel 1 is now tracking
            
            self.channel_1[1] = -1
            # Set T&H of inverted gate to sample
            cv6.voltage(sample)
            self.channel_2[1] = sample
            self.update_screen()


        @din.handler_falling
        def dinTriggerEnd():
            self.gate = False
            # Sample input
            sample = ain.read_voltage()
            # Set mirrored gate LOW
            cv1.off()
            # Set inverse gate HIGH
            cv4.on()

            # Set T&H of gate to sample
            cv3.voltage(sample)
            self.channel_1[1] = sample
            # Set S&H of inverted gate to sample
            cv5.voltage(sample)
            self.channel_2[0] = sample
            # Channel 2 is now tracking
            self.channel_2[1] = -1
            self.update_screen()

    def update(self):
        # Sample input
        sample = ain.read_voltage()
        if self.gate:
            # Pass sample to T&H
            cv3.voltage(sample)
        else:
            # Pass sample to inverted T&H 
            cv6.voltage(sample)

    def main(self):
        self.update_screen()
        while True:
            self.update()
            
    def update_screen(self):
        oled.fill(0)
        oled.text(f"Noddy v{VERSION}",0,0,1)
        oled.text("1",0,8,1)
        oled.text("2",0,16,1)

        if self.gate:
            oled.fill_rect(10,8,10,6,1)
        else:
            oled.fill_rect(10,16,10,6,1)
            
        oled.text(f"S:{self.channel_1[0]:.2f}",25,8,1)
        oled.text(f"S:{self.channel_2[0]:.2f}",25,16,1)

        if self.gate:
            oled.text(f"T:T",80,8,1)
            oled.text(f"T:{self.channel_2[1]:.2f}",80,16,1)
        else:
            oled.text(f"T:{self.channel_1[1]:.2f}",80,8,1)
            oled.text(f"T:T",80,16,1)
        oled.show()

if __name__ == "__main__":
    NoddyHolder().main()



from europi import *
import machine
from time import ticks_diff, ticks_ms
from random import randint, uniform

from europi_script import EuroPiScript

'''
Consequencer
author: Nik Ansell (github.com/gamecat69)
date: 2022-02-05
labels: sequencer, triggers, drums, randomness

A gate and CV sequencer inspired by Grids from Mutable Instruments that contains pre-loaded drum patterns that can be smoothly morphed from one to another. Triggers are sent from outputs 1 - 3, randomized stepped CV patterns are sent from outputs 4 - 6.
Send a clock to the digital input to start the sequence.

Demo video: https://youtu.be/UwjajP6uiQU

digital_in: clock in
analog_in: randomness CV

knob_1: randomness
knob_2: select pre-loaded drum pattern

button_1: Short Press: toggle randomized hi-hats on / off. Long Press: Play previous CV Pattern
button_2: Short PressL Generate a new random cv pattern for outputs 4 - 6. Long Press: Cycle through analogue input modes

output_1: trigger 1 / Bass Drum
output_2: trigger 2 / Snare Drum
output_3: trigger 3 / Hi-Hat
output_4: randomly generated CV (cycled by pushing button 2)
output_5: randomly generated CV (cycled by pushing button 2)
output_6: randomly generated CV (cycled by pushing button 2)

'''

class Consequencer(EuroPiScript):
    def __init__(self):
        # Overclock the Pico for improved performance.
        machine.freq(250_000_000)

        # Initialize sequencer pattern arrays   
        p = pattern()     
        self.BD=p.BD
        self.SN=p.SN
        self.HH=p.HH

        # Initialize variables
        self.step = 0
        self.trigger_duration_ms = 50
        self.clock_step = 0
        self.pattern = 0
        self.random_HH = False
        self.minAnalogInputVoltage = 0.9
        self.randomness = 0
        self.analogInputMode = 1 # 1: Randomness, 2: Pattern, 3: CV Pattern
        self.CvPattern = 0
        
        # Generate random CV for cv4-6
        self.random4 = []
        self.random5 = []
        self.random6 = []
        
        self.generateNewRandomCVPattern()

        # Triggered when button 2 is released.
        # Short press: Generate random CV for cv4-6
        # Long press: Change operating mode
        @b2.handler_falling
        def b2Pressed():
            
            if ticks_diff(ticks_ms(), b2.last_pressed()) >  300:
                if self.analogInputMode < 3:
                    self.analogInputMode += 1
                else:
                    self.analogInputMode = 1
            else:
                # Move to next cv pattern if one already exists, otherwise create a new one
                self.CvPattern += 1
                if self.CvPattern == len(self.random4):
                    self.generateNewRandomCVPattern()
            
        # Triggered when button 1 is released
        # Short press: Play previous CV pattern for cv4-6
        # Long press: Toggle random high-hat mode
        @b1.handler_falling
        def b1Pressed():
            if ticks_diff(ticks_ms(), b1.last_pressed()) >  300:
                self.random_HH = not self.random_HH
            else:
                # Play previous CV Pattern, unless we are at the first pattern
                if self.CvPattern != 0:
                    self.CvPattern -= 1

        # Triggered on each clock into digital input. Output triggers.
        @din.handler
        def clockTrigger():

            self.step_length = len(self.BD[self.pattern])
            
            # A pattern was selected which is shorter than the current step. Set to zero to avoid an error
            if self.step >= self.step_length:
                self.step = 0 

            # Set cv4-6 voltage outputs based on previously generated random pattern
            cv4.voltage(self.random4[self.CvPattern][self.step])
            cv5.voltage(self.random5[self.CvPattern][self.step])
            cv6.voltage(self.random6[self.CvPattern][self.step])

            # How much randomness to add to cv1-3
            # As the randomness value gets higher, the chance of a randomly selected int being lower gets higher
            if randint(0,99) < self.randomness:
                cv1.value(randint(0, 1))
                cv2.value(randint(0, 1))
                cv3.value(randint(0, 1))
            else:
                cv1.value(int(self.BD[self.pattern][self.step]))
                cv2.value(int(self.SN[self.pattern][self.step]))                    

                # If randomize HH is ON:
                if self.random_HH:
                    cv3.value(randint(0, 1))
                else:
                    cv3.value(int(self.HH[self.pattern][self.step]))

            # Reset clock step at 128 to avoid a HUGE integer if running for a long time
            # over a really long period of time this would look like a memory leak
            if self.clock_step < 128:
                self.clock_step +=1
            else:
                self.clock_step = 0
    
            # Reset step number at step_length -1 as pattern arrays are zero-based
            if self.step < self.step_length - 1:
                self.step += 1
            else:
                self.step = 0

        @din.handler_falling
        def clockTriggerEnd():
            cv1.off()
            cv2.off()
            cv3.off()

    def generateNewRandomCVPattern(self):
        self.step_length = len(self.BD[self.pattern])
        self.random4.append(self.generateRandomPattern(16, 0, 9))
        self.random5.append(self.generateRandomPattern(16, 0, 9))
        self.random6.append(self.generateRandomPattern(16, 0, 9))

    def getPattern(self):
        # If mode 2 and there is CV on the analogue input use it, if not use the knob position
        val = 100 * ain.percent()
        if self.analogInputMode == 2 and val > self.minAnalogInputVoltage:
            self.pattern = int((len(self.BD) / 100) * val)
        else:
            self.pattern = k2.read_position(len(self.BD))
        
        self.step_length = len(self.BD[self.pattern])

    def getCvPattern(self):
        # If analogue input mode 3, get the CV pattern from CV input
        if self.analogInputMode != 3:
            return
        else:
            # Get the analogue input voltage as a percentage
            CvpVal = 100 * ain.percent()
            
            # Is there a voltage on the analogue input and are we configured to use it?
            if CvpVal > 0.4:
                # Convert percentage value to a representative index of the pattern array
                self.CvPattern = int((len(self.random4) / 100) * CvpVal)

    def generateRandomPattern(self, length, min, max):
        self.t=[]
        for i in range(0, length):
            self.t.append(uniform(0,9))
        return self.t


    def getRandomness(self):
        # If mode 1 and there is CV on the analogue input use it, if not use the knob position
        val = 100 * ain.percent()
        if self.analogInputMode == 1 and val > self.minAnalogInputVoltage:
            self.randomness = val
        else:
            self.randomness = k1.read_position()

    def main(self):
        while True:
            self.getPattern()
            self.getRandomness()
            self.getCvPattern()
            self.updateScreen()
            self.reset_timeout = 500
            # If I have been running, then stopped for longer than reset_timeout, reset the steps and clock_step to 0
            if self.clock_step != 0 and ticks_diff(ticks_ms(), din.last_triggered()) > self.reset_timeout:
                self.step = 0
                self.clock_step = 0

    def visualizePattern(self, pattern):
        self.t = pattern
        self.t = self.t.replace('1','^')
        self.t = self.t.replace('0',' ')
        return self.t

    def updateScreen(self):
        #oled.clear() - dont use this, it causes the screen to flicker!
        oled.fill(0)
        
        # Show selected pattern visually
        oled.text(self.visualizePattern(self.BD[self.pattern]),0,0,1)
        oled.text(self.visualizePattern(self.SN[self.pattern]),0,10,1)
        oled.text(self.visualizePattern(self.HH[self.pattern]),0,20,1)

        # If the random toggle is on, show a rectangle
        if self.random_HH:
            oled.fill_rect(0,29,20,3,1)

        # Show the analogInputMode
        oled.text('M' + str(self.analogInputMode), 112, 25, 1)

        # Show randomness
        oled.text('R' + str(int(self.randomness)), 40, 25, 1)    

        # Show CV pattern
        oled.text('C' + str(self.CvPattern), 76, 25, 1)

        oled.show()

class pattern:

    # Initialize pattern lists
    BD=[]
    SN=[]
    HH=[]

    # Add patterns
    BD.append("1000100010001000")
    SN.append("0000000000000000")
    HH.append("0000000000000000")

    BD.append("1000100010001000")
    SN.append("0000000000000000")
    HH.append("0010010010010010")

    BD.append("1000100010001000")
    SN.append("0000100000000000")
    HH.append("0010010010010010")

    BD.append("1000100010001000")
    SN.append("0000100000001000")
    HH.append("0010010010010010")

    BD.append("1000100010001000")
    SN.append("0000100000000000")
    HH.append("0000000000000000")

    BD.append("1000100010001000")
    SN.append("0000100000001000")
    HH.append("0000000000000000")

    BD.append("1000100010001000")
    SN.append("0000100000001000")
    HH.append("0000100010001001")

    BD.append("1000100010001000")
    SN.append("0000100000001000")
    HH.append("0101010101010101")

    BD.append("1000100010001000")
    SN.append("0000000000000000")
    HH.append("1111111111111111")

    BD.append("1000100010001000")
    SN.append("0000100000001000")
    HH.append("1111111111111111")

    BD.append("1000100010001000")
    SN.append("0000100000000000")
    HH.append("0001000000000000")

    BD.append("1000100010001000")
    SN.append("0000100000000000")
    HH.append("0001001000000000")

    # Source: https://docs.google.com/spreadsheets/d/19_3BxUMy3uy1Gb0V8Wc-TcG7q16Amfn6e8QVw4-HuD0/edit#gid=0
    BD.append("1000000010000000")
    SN.append("0000100000001000")
    HH.append("1010101010101010")

    BD.append("1010001000100100")
    SN.append("0000100101011001")
    HH.append("0000000100000100")

    BD.append("1000000110000010")
    SN.append("0000100000001000")
    HH.append("1010101110001010")

    BD.append("1100000100110000")
    SN.append("0000100000001000")
    HH.append("1010101010101010")

    BD.append("1000000110100000")
    SN.append("0000100000001000")
    HH.append("0010101010101010")

    BD.append("1010000000110001")
    SN.append("0000100000001000")
    HH.append("1010101010101010")

    BD.append("1000000110100001")
    SN.append("0000100000001000")
    HH.append("0000100010101011")

    BD.append("1001001010000000")
    SN.append("0000100000001000")
    HH.append("0000100000001000")

    BD.append("1010001001100000")
    SN.append("0000100000001000")
    HH.append("1010101010001010")

    BD.append("1010000101110001")
    SN.append("0000100000001000")
    HH.append("1010101010001010")

    # End external patterns

    BD.append("1000100010001000")
    SN.append("0000101001001000")
    HH.append("0101010101010101")

    BD.append("1100000001010000")
    SN.append("0000101000001000")
    HH.append("0101010101010101")

    BD.append("1100000001010000")
    SN.append("0000101000001000")
    HH.append("1111111111111111")

    BD.append("1001001001000100")
    SN.append("0001000000010000")
    HH.append("0101110010011110")

    BD.append("1001001001000100")
    SN.append("0001000000010000")
    HH.append("1111111111111111")

    # Be warned patterns < 16 steps can sound disjointed when using CV to select the pattern!

    BD.append("10010000010010")
    SN.append("00010010000010")
    HH.append("11100110111011")

    BD.append("1001000001001")
    SN.append("0001001000001")
    HH.append("1110011011101")

    BD.append("100100000100")
    SN.append("000100100000")
    HH.append("111001101110")

    BD.append("10010000010")
    SN.append("00010010000")
    HH.append("11100110111")

    BD.append("10010000010")
    SN.append("00010010000")
    HH.append("11111010011")

    BD.append("1001000010")
    SN.append("0001000000")
    HH.append("1111101101")

    BD.append("100100010")
    SN.append("000100000")
    HH.append("111110111")

    BD.append("10010010")
    SN.append("00010000")
    HH.append("11111111")

    BD.append("1001001")
    SN.append("0001000")
    HH.append("1111111")

    BD.append("100100")
    SN.append("000100")
    HH.append("111111")

    BD.append("10000")
    SN.append("00001")
    HH.append("11110")

    BD.append("1000")
    SN.append("0000")
    HH.append("1111")

    BD.append("100")
    SN.append("000")
    HH.append("111")


if __name__ == '__main__':
    # Reset module display state.
    oled.clear()
    [cv.off() for cv in cvs]
    dm = Consequencer()
    dm.main()



"""See menu.md for details."""
from bootloader import BootloaderMenu

from contrib.coin_toss import CoinToss
from contrib.consequencer import Consequencer
from contrib.diagnostic import Diagnostic
from contrib.harmonic_lfos import HarmonicLFOs
from contrib.hello_world import HelloWorld
from contrib.noddy_holder import NoddyHolder
from contrib.polyrhythmic_sequencer import PolyrhythmSeq
from contrib.radio_scanner import RadioScanner
from contrib.scope import Scope
from calibrate import Calibrate

# Scripts that are included in the menu
EUROPI_SCRIPT_CLASSES = [
    CoinToss,
    Consequencer,
    Diagnostic,
    HarmonicLFOs,
    HelloWorld,
    NoddyHolder,
    PolyrhythmSeq,
    RadioScanner,
    Scope,
    Calibrate,
]


if __name__ == "__main__":
    BootloaderMenu(EUROPI_SCRIPT_CLASSES).main()



from setuptools import setup

setup(
    name="micropython-europi-contrib",
    version="0.3.0",
    description="Community Contributions to EuroPi module for MicroPython",
    long_description="Community Contributions to EuroPi module for MicroPython",
    url="https://github.com/Allen-Synthesis/EuroPi",
    author="Allen Synthesis",
    author_email="contact@allensynthesis.co.uk",
    license="Apache 2.0",
    packages=["contrib"],
    namespace_packages=["contrib"],
)



from time import sleep
from europi import oled, b1, din
from europi_script import EuroPiScript


class HelloWorld(EuroPiScript):
    def __init__(self):
        super().__init__()
        state = self.load_state_json()

        self.counter = state.get("counter", 0)
        self.enabled = state.get("enabled", True)

        din.handler(self.increment_counter)
        b1.handler(self.toggle_enablement)

    @classmethod
    def display_name(cls):
        return "Hello World"

    def increment_counter(self):
        if self.enabled:
            self.counter += 1
            self.save_state()

    def toggle_enablement(self):
            self.enabled = not self.enabled
            self.save_state()

    def save_state(self):
        """Save the current state variables as JSON."""
        # Don't save if it has been less than 5 seconds since last save.
        if self.last_saved() < 5000:
            return

        state = {
            "counter": self.counter,
            "enabled": self.enabled,
        }
        self.save_state_json(state)

    def main(self):
        while True:
            oled.centre_text(f"Hello world\n{self.counter}")
            sleep(1)

if __name__ == "__main__":
    HelloWorld().main()



from europi import *
from math import cos, radians
from random import randint
from time import sleep
from europi_script import EuroPiScript

MAX_VOLTAGE = 10
HARMONICS = [1, 3, 5, 7, 11, 13]

def reset():
    global degree
    degree = 0

def change_mode():
    global MODE
    MODE += 1
    
    if MODE == 3:
        MODE = 0

def get_delay_increment_value_random_chance():
    random_chance = k2.read_position(100,1)
    delay = (0.1 - (k1.read_position(100)/1000)) + (ain.read_voltage(1)/100)
    return delay, round((((1/delay)-10)/1)+1), random_chance

def change_harmonic():
    global HARMONICS
    HARMONICS[randint(0,5)] = randint(1,13)

MODE = 0
degree = 0
delay, increment_value, random_chance = get_delay_increment_value_random_chance()
pixel_x = OLED_WIDTH-1
pixel_y = OLED_HEIGHT-1

class HarmonicLFOs(EuroPiScript):

    def main(self):
        global degree, delay, increment_value, random_chance

        din.handler(reset)
        b1.handler(reset)
        b2.handler(change_mode)

        while True:
            rad = radians(degree)
            
            if randint(0,100) < random_chance:
                change_harmonic()
            
            oled.vline(pixel_x,0,OLED_HEIGHT,0)
            for cv, multiplier in zip(cvs, HARMONICS):
                if MODE == 0: #Sin
                    volts = ((0-(cos(rad*(1/multiplier)))+1))*(MAX_VOLTAGE/2)
                elif MODE == 1: #Saw
                    volts = ((degree%(360*multiplier))/(360*multiplier))*MAX_VOLTAGE
                elif MODE == 2: #Square
                    volts = MAX_VOLTAGE * (int(((degree%(360*multiplier))/(360*multiplier))*MAX_VOLTAGE) < (MAX_VOLTAGE/2))
                    
                cv.voltage(volts)
                if cv != cv1:
                    oled.pixel(pixel_x,pixel_y-int(volts*(pixel_y/10)),1)
            
            degree += increment_value
            sleep(delay)
            oled.scroll(-1,0)
            
            if round(degree, -1) % 10 == 0:
                delay, increment_value, random_chance = get_delay_increment_value_random_chance()
                oled.show()


if __name__ == "__main__":
    HarmonicLFOs().main()